require_gem "sap4rails"

class Configuration < SAP4Rails::Base

  function_module :CUX1_CFG_NEW,
    :CUX1_CHARACTS_READ,
    :CUX1_CHARACTS_VALS_READ,
    :CAVC_I_CHARS_SET_VALUES,
    :CAVC_I_CHARS_GET_ASSIGNED_VALS,
    :CAVC_I_CHARS_GET_VALID_VALS,
    :SD_SALESDOCUMENT_CREATE

  def self.start(material)
    @material = material
    Configuration.CUX1_CFG_NEW.reset
    Configuration.CUX1_CFG_NEW.ROOT_NAME.value = material
    Configuration.CUX1_CFG_NEW.DATE.value = '20061113'
    Configuration.CUX1_CFG_NEW.call
    @objectinstance = Configuration.CUX1_CFG_NEW.ROOT_OBJECT_INSTANCE.value
  end

  def self.characts
    # Read Characteristics
    Configuration.CUX1_CHARACTS_READ.reset
    Configuration.CUX1_CHARACTS_READ.INSTANCE.value = @objectinstance
    Configuration.CUX1_CHARACTS_READ.call
    # Read Values of Characteristics
#    Configuration.CUX1_CHARACTS_VALS_READ.reset
#    Configuration.CUX1_CHARACTS_VALS_READ.INSTANCE.value = @objectinstance
#    Configuration.CUX1_CHARACTS_VALS_READ.call
#    if Configuration.CUX1_CHARACTS_VALS_READ.CHARACTS_VALS != nil
#      ret = "S"
#      characts_values = []
#      Configuration.CUX1_CHARACTS_VALS_READ.CHARACTS_VALS.hashRows do |r|
#        characts_values << r
#      end
#    end
    # Read Valid Values of Characteristics
    Configuration.CAVC_I_CHARS_GET_VALID_VALS.reset
    Configuration.CAVC_I_CHARS_GET_VALID_VALS.INSTANCE.value = @objectinstance
    Configuration.CAVC_I_CHARS_GET_VALID_VALS.call
    if Configuration.CAVC_I_CHARS_GET_VALID_VALS.CHARACTERISTICS_VALUE != nil
      characts_values = []
      Configuration.CAVC_I_CHARS_GET_VALID_VALS.CHARACTERISTICS_VALUE.hashRows do |r|
        characts_values << r
      end
    end
    # Build Array
    if Configuration.CUX1_CHARACTS_READ.CHARACTS_ATTRIBUTES != nil
      ret = "S"
      characts = []
      Configuration.CUX1_CHARACTS_READ.CHARACTS_ATTRIBUTES.hashRows do |r|
        characts << r
      end
      characts_and_values = []
      Configuration.CUX1_CHARACTS_READ.CHARACTS_ATTRIBUTES.hashRows do |c|
        values = []
        characts_values.each do |v|
          #if v['ATNAM'] == c['ATNAM']
          #  values << v
          #end
          if v['CHARACT'] == c['ATNAM']
            values << v
          end
        end
        c['VALUES'] = values
        characts_and_values << c
      end
    else
      ret = "E"
    end
    return ret, characts, characts_values, characts_and_values
  end

  def self.check(values)
    # Set Characteristics
    Configuration.CAVC_I_CHARS_SET_VALUES.reset
    Configuration.CAVC_I_CHARS_SET_VALUES.INSTANCE.value = @objectinstance
		charvalues = Configuration.CAVC_I_CHARS_SET_VALUES.CHARACTERISTIC_VALUE.structure
    values.each do |key, value|
      # Configuration for 5LS4
		  charvalues.charact.value = key.to_s
  		charvalues.VALUE.value = value.to_s
      Configuration.CAVC_I_CHARS_SET_VALUES.CHARACTERISTIC_VALUE.value = [ charvalues.value ]
    end
    Configuration.CAVC_I_CHARS_SET_VALUES.call
    error_characteristic_value = Configuration.CAVC_I_CHARS_SET_VALUES.ERROR_CHARACTERISTIC_VALUE.value
    ret = Configuration.CAVC_I_CHARS_SET_VALUES.RETURN.value
    # Get Assigned Values
    Configuration.CAVC_I_CHARS_GET_ASSIGNED_VALS.reset
    Configuration.CAVC_I_CHARS_GET_ASSIGNED_VALS.INSTANCE.value = @objectinstance
    Configuration.CAVC_I_CHARS_GET_ASSIGNED_VALS.call
    assigned_values = Configuration.CAVC_I_CHARS_GET_ASSIGNED_VALS.CHARACTERISTICS_VALUE
    # Simulate Salesorder
    Configuration.SD_SALESDOCUMENT_CREATE.TESTRUN.value = 'X'
    Configuration.SD_SALESDOCUMENT_CREATE.BUSINESS_OBJECT = 'BUS2031'
    date = '20061127'
    # Order Header
		orderheader = Configuration.SD_SALESDOCUMENT_CREATE.SALES_HEADER_IN.structure

		# orderheader.DOC_TYPE.value = 'ZAG'
		orderheader.DOC_TYPE.value = 'AG'
		orderheader.SALES_ORG.value = '1000'
		orderheader.DISTR_CHAN.value = '10'
		# orderheader.DIVISION.value = '10'
		orderheader.DIVISION.value = '00'
		orderheader.QT_VALID_T.value = date

		Configuration.SD_SALESDOCUMENT_CREATE.SALES_HEADER_IN.value = orderheader.value

		# Order Items
		orderitems = Configuration.SD_SALESDOCUMENT_CREATE.SALES_ITEMS_IN.structure
		orderitems.ITM_NUMBER.value = '000010'
		orderitems.PO_ITM_NO.value = orderitems.ITM_NUMBER.value
		orderitems.MATERIAL.value = @material
		Configuration.SD_SALESDOCUMENT_CREATE.SALES_ITEMS_IN.value = [ orderitems.value ]
		# Order Schedule
		orderschedule = Configuration.SD_SALESDOCUMENT_CREATE.SALES_SCHEDULES_IN.structure
		orderschedule.ITM_NUMBER.value = orderitems.ITM_NUMBER.value
		orderschedule.REQ_QTY.value = '1'
		Configuration.SD_SALESDOCUMENT_CREATE.SALES_SCHEDULES_IN.value = [ orderschedule.value ]
		# Configuration
		# Reference Data
		cfgsref = Configuration.SD_SALESDOCUMENT_CREATE.SALES_CFGS_REF.structure
		cfgsref.POSEX.value = orderitems.ITM_NUMBER.value
		cfgsref.CONFIG_ID.value = '000001'
		cfgsref.ROOT_ID.value = '00000001'
		Configuration.SD_SALESDOCUMENT_CREATE.SALES_CFGS_REF.value = [ cfgsref.value ]
		# Instances
		cfgsinst = Configuration.SD_SALESDOCUMENT_CREATE.SALES_CFGS_INST.structure
		cfgsinst.CONFIG_ID.value = '000001'
		cfgsinst.INST_ID.value = '00000001'
		cfgsinst.OBJ_TYPE.value = 'MARA'
		cfgsinst.CLASS_TYPE.value = '300'
		cfgsinst.OBJ_KEY.value = @material
		cfgsinst.QUANTITY.value = '1'
		cfgsinst.QUANTITY_UNIT.value = 'PC'
		cfgsinst.COMPLETE.value = 'T'
		cfgsinst.CONSISTENT.value = 'T'
		cfgsinst.PERSIST_ID_TYPE.value = 'G'
		Configuration.SD_SALESDOCUMENT_CREATE.SALES_CFGS_INST.value = [ cfgsinst.value ]
		# Characteristic Values
		cfgsvalue = Configuration.SD_SALESDOCUMENT_CREATE.SALES_CFGS_VALUE.structure
		cfgsvalue.CONFIG_ID.value = '000001'
		cfgsvalue.INST_ID.value = '00000001'
		cfgsvalue.VALCODE.value = '1'
		counter = 0
		values.each do |key, value|
			cfgsvalue.CHARC.value = key.to_s
			cfgsvalue.VALUE.value = value.to_s
			Configuration.SD_SALESDOCUMENT_CREATE.SALES_CFGS_VALUE.value.push( cfgsvalue.value )
		end
		# Order Partners
		orderpartners = Configuration.SD_SALESDOCUMENT_CREATE.SALES_PARTNERS.structure
		orderpartners.PARTN_ROLE.value = 'AG'
		# orderpartners.PARTN_NUMB.value = '0001003258'
		orderpartners.PARTN_NUMB.value = '0000001555'
		Configuration.SD_SALESDOCUMENT_CREATE.SALES_PARTNERS.value = [ orderpartners.value ]
		# Call Function
		Configuration.SD_SALESDOCUMENT_CREATE.call
		ret_salesorder_simulate = Configuration.SD_SALESDOCUMENT_CREATE.RETURN
    return ret, error_characteristic_value, assigned_values, ret_assigned, ret_salesorder_simulate
  end
end
