require 'test/unit'
require 'test/unit/assertions'
require 'rubygems'
require_gem 'saprfc'

class SAPTest < Test::Unit::TestCase

	# Login Data
	@@partneremployeeid = "25070"
	@@password = "1234567890"
	@@newpassword = "test2"
	# Product Data
	@@material = 'P-100'
	# @@confmaterial = 'P-500'
	# @@confmaterial = 'DPC1'
	@@confmaterial = '5LS4'
	@@unit = ''
	# Values set during Test
	@@customer = ''
	@@salesorg = ''
	@@subtotal1 = ''
	@@subtotal2 = ''

	def setup
		SAP::Rfc.useCache = true
		@rfc = SAP::Rfc.new(
									 :ashost => "der304.siteco.net",
                   :sysnr  => 64,
                   :lang   => "DE",
                   :client => "002",
                   :user   => "ITS",
                   :passwd => "test1",
                   :trace  => 0
									 )
	end

	def test_CSC_00005_Login
		# Check Password
		pwcheck = @rfc.discover("BAPI_PAR_EMPLOYEE_CHECKPASSWOR")
		pwcheck.PARTNEREMPLOYEEID.value = @@partneremployeeid
		pwcheck.PASSWORD.value = @@password
		@rfc.call(pwcheck)
		ret = pwcheck.RETURN.value
		assert(ret['TYPE'] == 'S')
		# Get Customer Number
		excheck = @rfc.discover("BAPI_PAR_EMPLOYEE_CHECKEXISTEN")
		excheck.PARTNEREMPLOYEEID.value = @@partneremployeeid
		@rfc.call(excheck)
		@@customer = excheck.CUSTOMER.value
		assert( @@customer != nil)
		#puts "\nCustomer Number: " + @@customer
		# Get Sales Areas
		getsalesareas = @rfc.discover("BAPI_CUSTOMER_GETSALESAREAS")
		getsalesareas.CUSTOMERNO.value = @@customer
		@rfc.call(getsalesareas)
		ret = getsalesareas.RETURN.value
		assert(ret['TYPE'] == 'S')
		getsalesareas.SALESAREAS.hashRows {
		  |x|
		  @@salesorg = x['SALESORG']
		  # puts @@salesorg
		}
		assert(@@salesorg == '8000')
		@@salesorg = '1000'
	end

	def test_CSC_00100_StartConfiguration
		newcfg = @rfc.discover("CUX1_CFG_NEW")
		newcfg.ROOT_NAME.value = @@confmaterial
		newcfg.DATE = '20061113'
		# Call Function
		@rfc.call(newcfg)
		assert( newcfg.new_cfg_pointer.value != nil )
		assert( newcfg.root_object_instance.value != nil )
		cfgpointer = newcfg.new_cfg_pointer.value
		objectinstance = newcfg.root_object_instance.value
		#puts "\nCFG Pointer: " + newcfg.new_cfg_pointer.value
		#puts "Instance: " + newcfg.root_object_instance.value
		# Read Charactaristics
		characts = @rfc.discover("CUX1_CHARACTS_READ")
		characts.instance.value = objectinstance
		@rfc.call(characts)
		# puts "characts: "
		characts.characts.hashRows {
			|x|
			# puts x['ATINN'] + " " + x['ATNAM'] + " " + x['ATBEZ'] + x['FLREQ'] + x['FLINH'] + x['FLASS']
		}
		#puts "characts_attributes: "
		#characts.characts_attributes.hashRows {
		#	|x|
		#	puts x['ATINN'] + x['ATNAM'] + x['ATBEZ'] + x['ATFOR'] + x['ATEIN'] + x['ATERF']
		#}
		# Read Charactaristics
		charactsvalues = @rfc.discover("CUX1_CHARACTS_VALS_READ")
		charactsvalues.instance.value = objectinstance
		@rfc.call(charactsvalues)
		#puts "\ncharacts_vals: "
		#charactsvalues.characts_vals.hashRows {
		#	|x|
		#	puts x['ATINN'] +";"+ x['ATNAM'] +";"+ x['ATBEZ'] +";"+ x['ATWRT']+";"+ x['ATWTB'] + ";"+x['ATFLV'] +";"+ x['ATAWE'] +";"+ x['ATFLB'] +";"+ x['ATAW1'] +";"+ x['ATCOD']+";"+x['ATTLV'] +";"+ x['ATTLB'] +";"+ x['ATPRZ'] +";"+ x['ATINC'] +";"+ x['KNOBJ']+";"+x['DOKAR'] +";"+ x['DOKNR'] +";"+ x['DOKVR'] +";"+ x['DOKTL']
		#}
		# Read allowed values
		allowedvalues = @rfc.discover("CAVC_I_CHARS_GET_ALLOWED_VALS")
		allowedvalues.instance.value = objectinstance
		@rfc.call(allowedvalues)
		#puts "\nallowedvalues: "
		#allowedvalues.characteristics_value.hashRows {
		#	|x|
		#	puts x['CHARACT']+";"+x['CH_DESCR']+";"+x['VALUE'] +";"+ x['VAL_DESCR']+";"+ x['VAL_DEFAUL']+";"+ x['VAL_VALID']+";"+ x['VAL_ASS']
		#}
		# Read valid values
		validvalues = @rfc.discover("CAVC_I_CHARS_GET_VALID_VALS")
		validvalues.instance.value = objectinstance
		@rfc.call(validvalues)
		#puts "\nvalidvalues: "
		#validvalues.characteristics_value.hashRows {
		#	|x|
		#	puts x['CHARACT']+";"+x['CH_DESCR']+";"+x['VALUE'] +";"+ x['VAL_DESCR']+";"+ x['VAL_DEFAUL']+";"+ x['VAL_VALID']+";"+ x['VAL_ASS']
		#}
		# Set values
		setvalues = @rfc.discover("CAVC_I_CHARS_SET_VALUES")
		setvalues.instance.value = objectinstance
		charvalues = setvalues.characteristic_value.structure
# Configuration for 5LS4
		charvalues.charact.value = 'KFG_LS4_WANNENTYP'
		charvalues.VALUE.value = '1' # PMMA-Wanne
		setvalues.characteristic_value.value = [ charvalues.value ]
		charvalues.charact.value = 'KFG_LS4_SCHUTZKLASSE'
		charvalues.VALUE.value = '2' # SK I, V2A Verschlsse montiert
		setvalues.characteristic_value.value.push( charvalues.value )
		charvalues.charact.value = 'KFG_LS4_VG'
		charvalues.VALUE.value = 'D' # Dali
		setvalues.characteristic_value.value.push( charvalues.value )
		charvalues.charact.value = 'KFG_LS4_LAMPIGKEIT'
		charvalues.VALUE.value = '2' # 2-Lampig
		setvalues.characteristic_value.value.push( charvalues.value )
		charvalues.charact.value = 'KFG_LS4_LEISTUNG'
		charvalues.VALUE.value = 'E' # 58 W
		setvalues.characteristic_value.value.push( charvalues.value )

# Configuration for DPC1
=begin
		charvalues.charact.value = 'DPCX_COUNTRY'
		charvalues.VALUE.value = '001'
		setvalues.characteristic_value.value = [ charvalues.value ]
		charvalues.charact.value = 'DPCX_CASE'
		charvalues.VALUE.value = '002'
		setvalues.characteristic_value.value.push( charvalues.value )
		charvalues.charact.value = 'DPCX_CPU'
		charvalues.VALUE.value = '003'
		setvalues.characteristic_value.value.push( charvalues.value )
		charvalues.charact.value = 'DPCX_RAM_QUANTITY'
		charvalues.VALUE.value = '256'
		setvalues.characteristic_value.value.push( charvalues.value )
		charvalues.charact.value = 'DPCX_HDD'
		charvalues.VALUE.value = '003'
		setvalues.characteristic_value.value.push( charvalues.value )
		charvalues.charact.value = 'DPCX_MONITOR'
		charvalues.VALUE.value = '002'
		setvalues.characteristic_value.value.push( charvalues.value )
=end
# Configuration for P-500
=begin
		charvalues.charact.value = 'P_01'
		charvalues.VALUE.value = '02'
		setvalues.characteristic_value.value = [ charvalues.value ]
		charvalues.charact.value = 'P_02'
		charvalues.VALUE.value = '02'
		setvalues.characteristic_value.value.push( charvalues.value )
		charvalues.charact.value = 'P_03'
		charvalues.VALUE.value = '02'
		setvalues.characteristic_value.value.push( charvalues.value )
		charvalues.charact.value = 'P_06'
		charvalues.VALUE.value = '01'
		setvalues.characteristic_value.value.push( charvalues.value )
		charvalues.charact.value = 'P_08'
		charvalues.VALUE.value = '01'
		setvalues.characteristic_value.value.push( charvalues.value )
=end
		@rfc.call(setvalues)
		puts "\nerror_characteristic_value: "
		error = setvalues.error_characteristic_value.value
		puts error['CHARACT'] + error['VALUE'] + error['VAL_DESCR']
		puts "\nReturn: "
		ret = setvalues.return.value
		puts ret['MESSAGE']
		# Get assigned values
		getvalues = @rfc.discover("CAVC_I_CHARS_GET_ASSIGNED_VALS")
		getvalues.instance.value = objectinstance
		@rfc.call(getvalues)
		puts "\nassignedvalues: "
		getvalues.characteristics_value.hashRows {
			|x|
			# x['CHARACT']+";"+x['VALUE'] +";"+
			puts x['CH_DESCR']+";"+ x['VAL_DESCR']+";"+ x['VAL_DEFAUL']+";"+ x['VAL_VALID']+";"+ x['VAL_ASS']
		}
		# Read valid values
		validvalues = @rfc.discover("CAVC_I_CHARS_GET_VALID_VALS")
		validvalues.instance.value = objectinstance
		@rfc.call(validvalues)
		#puts "\nvalidvalues: "
		#validvalues.characteristics_value.hashRows {
		#	|x|
		#	puts x['CHARACT']+";"+x['CH_DESCR']+";"+x['VALUE'] +";"+ x['VAL_DESCR']+";"+ x['VAL_DEFAUL']+";"+ x['VAL_VALID']+";"+ x['VAL_ASS']
		#}
		salesordersimulate = @rfc.discover("SD_SALESDOCUMENT_CREATE")
		salesordersimulate.TESTRUN.value = 'X'
		salesordersimulate.BUSINESS_OBJECT = 'BUS2031'
		date = '20050830'
		# Order Header
		orderheader = salesordersimulate.SALES_HEADER_IN.structure
		orderheader.DOC_TYPE.value = 'ZAG'
		orderheader.SALES_ORG.value = @@salesorg
		orderheader.DISTR_CHAN.value = '10'
		orderheader.DIVISION.value = '10'

		orderheader.QT_VALID_T.value = date
		salesordersimulate.SALES_HEADER_IN.value = orderheader.value
		# Order Items
		orderitems = salesordersimulate.SALES_ITEMS_IN.structure
		orderitems.itm_number.value = '000010'
		orderitems.PO_ITM_NO.value = orderitems.itm_number.value
		orderitems.MATERIAL.value = @@confmaterial
		# orderitems.TARGET_QTY.value = '1'
		salesordersimulate.SALES_ITEMS_IN.value = [ orderitems.value ]
		# Order Schedule
		orderschedule = salesordersimulate.SALES_SCHEDULES_IN.structure
		orderschedule.ITM_NUMBER.value = orderitems.itm_number.value
		orderschedule.REQ_QTY.value = '1'
		salesordersimulate.SALES_SCHEDULES_IN.value = [ orderschedule.value ]
		# Configuration
		# Reference Data
		cfgsref = salesordersimulate.SALES_CFGS_REF.structure
		cfgsref.POSEX.value = orderitems.itm_number.value
		cfgsref.CONFIG_ID.value = '000001'
		cfgsref.ROOT_ID.value = '00000001'
		salesordersimulate.SALES_CFGS_REF.value = [ cfgsref.value ]
		# Instances
		cfgsinst = salesordersimulate.SALES_CFGS_INST.structure
		cfgsinst.CONFIG_ID.value = '000001'
		cfgsinst.INST_ID.value = '00000001'
		cfgsinst.OBJ_TYPE.value = 'MARA'
		cfgsinst.CLASS_TYPE.value = '300'
		cfgsinst.OBJ_KEY.value = @@confmaterial
		cfgsinst.QUANTITY.value = '1'
		cfgsinst.QUANTITY_UNIT.value = 'PC'
		cfgsinst.COMPLETE.value = 'T'
		cfgsinst.CONSISTENT.value = 'T'
		cfgsinst.PERSIST_ID_TYPE.value = 'G'
		salesordersimulate.SALES_CFGS_INST.value = [ cfgsinst.value ]
		# Characteristic Values
		cfgsvalue = salesordersimulate.SALES_CFGS_VALUE.structure
		cfgsvalue.CONFIG_ID.value = '000001'
		cfgsvalue.INST_ID.value = '00000001'
		cfgsvalue.VALCODE.value = '1'
		counter = 0
		getvalues.characteristics_value.hashRows {
			|x|
			cfgsvalue.CHARC.value = x['CHARACT']
			cfgsvalue.VALUE.value = x['VALUE']
			salesordersimulate.SALES_CFGS_VALUE.value.push( cfgsvalue.value )
		}
		salesordersimulate.SALES_CFGS_VALUE.hashRows {
			|x|
			puts x['CONFIG_ID']+" "+x['INST_ID']+" "+x['VALCODE']+" "+x['CHARC']+" "+x['VALUE']
		}
		# Order Partners
		orderpartners = salesordersimulate.SALES_PARTNERS.structure
		orderpartners.PARTN_ROLE.value = 'AG'
		orderpartners.PARTN_NUMB.value = @@customer
		salesordersimulate.SALES_PARTNERS.value = [ orderpartners.value ]
		# Call Function
		@rfc.call(salesordersimulate)
		puts "Return Messages: "
		salesordersimulate.RETURN.hashRows {
			|x|
			puts x['MESSAGE']
		}
		puts "Subtotals: "
		salesordersimulate.ITEMS_EX.hashRows {
			|x|
			puts x['SUBTOT_PP1']+" "+x['SUBTOT_PP2']+" "+x['SUBTOT_PP3']
		}
		#puts "Conditions: "
		#salesordersimulate.CONDITIONS_EX.hashRows {
		#	|x|
		#	puts x['ITM_NUMBER']+" "+x['COND_ST_NO']+" "+x['COND_COUNT']+" "+x['COND_TYPE']+" "+x['COND_VALUE']+" "+x['CURRENCY']+" "+x['COND_UNIT']
		#}
	end

	def teardown
		@rfc.close()
	end
end
